//header file for core ROS functions
#include "ros/ros.h"

//header file for geometric transformations
#include <tf/tf.h>
#include <tf/transform_datatypes.h>

//header file for a ROS pose which encodes position and orientation
#include "geometry_msgs/PoseStamped.h"

//some standard C++ header file for input output
#include <stdio.h>
#include <iostream>

#define PI 3.14159265359

int main(int argc, char **argv){
	
	
	ros::init(argc, argv, "pose_tutorial");
	ros::NodeHandle n;
	
	//publisher for the global frame of reference pose
	ros::Publisher pose_pub_global = n.advertise<geometry_msgs::PoseStamped>("/frame_of_reference", 1);

	//publishes the poses generate after entering roll pitch yaw
	ros::Publisher pose_pub = n.advertise<geometry_msgs::PoseStamped>("/my_pose", 1);

	int counter = 0;

	while (ros::ok()){
		float roll, pitch, yaw;
		
		
		ROS_INFO("Enter roll (degrees):");
		std::cin >> roll;
		ROS_INFO("Enter pitch (degrees):");
		std::cin >> pitch;
		ROS_INFO("Enter yaw (degrees):");
		std::cin >> yaw;
		
		ROS_INFO("You entered: %f, %f, %f",roll,pitch,yaw);
		
		//convert degrees to radians
		roll = (roll / 360.0) * (2*PI);
		pitch = (pitch / 360.0) * (2*PI);
		yaw = (yaw / 360.0) * (2*PI);
		
		
		geometry_msgs::PoseStamped next_pose;
		next_pose.pose.position.x = 2.0;
		next_pose.pose.position.y = 2.0;
		next_pose.pose.position.z = 2.0;
		
		//the following function converts roll-pitch-yaw to a quaternion
		next_pose.pose.orientation = tf::createQuaternionMsgFromRollPitchYaw(roll,pitch,yaw);
		
		//to show the pose in rviz, it needs a header with a frame_id
		next_pose.header.seq = counter;
		counter ++;
		next_pose.header.frame_id = "/map";
		
		pose_pub.publish(next_pose);
		
		//also, publish the global frame of reference marker which is the same everytime
		geometry_msgs::PoseStamped pose_global;
		pose_global.header.frame_id = "/map";
		pose_global.pose.orientation = tf::createQuaternionMsgFromRollPitchYaw(0,0,0);
		pose_pub_global.publish(pose_global);
		
		
		
		//now, convert quaternion back to roll pitch yaw
		
		//create an instance tf::Quaternion object
		tf::Quaternion q(next_pose.pose.orientation.x, 
							next_pose.pose.orientation.y, 
							next_pose.pose.orientation.z, 
							next_pose.pose.orientation.w);
		
		//convert the qauternion to a 3 x 3 rotation matrix
		tf::Matrix3x3 m(q);
		
		//extracy roll pitch yaw from the matrix
		double r, p, y;
		m.getRPY(r, p, y);
		std::cout << "Roll: " << r << ", Pitch: " << p << ", Yaw: " << y << std::endl;
	}

/*
	geometry_msgs::Twist v;
	
	v.linear.x = 2.0;
	v.linear.y = 0.0;
	v.linear.z = 0.0;
	v.angular.x = 0.0;
	v.angular.y = 0.0;
	v.angular.z = -1.0;

	float duration = 3.0;
	float framerate = 40.0;
	ros::Rate loop_rate(framerate);
	
	double angular_z = 0.25;
	
	for (float t = 0; t < duration; t += 1.0/framerate){
		
		v.angular.z = angular_z;
		velocity_pub.publish(v);
	
		v.linear.x += 1.0;
		loop_rate.sleep();
	}
	
	
	
	*/
	
	
	
	return 0;
}
